/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import java.awt.event.*;
import java.util.*;

class RemoteSketchPanel 
    extends SketchPanel implements RemoteSyncListener
{
    /* states of the root BeaconConfigApp */
    private RemoteSyncEvent currentStates;
    private EventListener eventHandler;
    private boolean override;

    public RemoteSketchPanel(SolverPanel sp, EventListener e, boolean override)
    {
	super(sp);
	currentStates = new RemoteSyncEvent();
	currentStates.sketchPanelStates = super.getStates();
	eventHandler = e;
	this.override = override;
    }

    private void updateRemote()
    {
	if(!override) {
	    currentStates = new RemoteSyncEvent();
	    currentStates.sketchPanelStates = super.getStates();
	}
	if(eventHandler instanceof RemoteSyncListener) {
	    // calls handler in NetworkServer
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
	}
    }

    public void enableButtons(boolean v)
    {
	super.enableButtons(v);
	updateRemote();
    }

    public void actionPerformed(ActionEvent e) 
    {
	Util.dbgPrintln("RemoteSketchPanel::actionPerformed "+
			e.getActionCommand());
	if(!override) { // root app
	    super.actionPerformed(e);
	    currentStates = new RemoteSyncEvent();
	    currentStates.sketchPanelStates = super.getStates();
	}
	else {
	    // remote app: restore old states, then wait for ack from
	    // root application
	    setCurrentStates(currentStates);
	}

	if(eventHandler instanceof ActionListener)
	    ((ActionListener) eventHandler).actionPerformed(e);
	else if(eventHandler instanceof RemoteSyncListener)
	    // calls handler in NetworkServer
	    ((RemoteSyncListener) eventHandler).remoteSyncEvent(currentStates);
    }

    private void setCurrentStates(RemoteSyncEvent cs)
    {
	Util.dbgPrintln("RemoteSketchPanel::setCurrentStates");
	currentStates = cs;
	if(cs.sketchPanelStates != null) {
	    Boolean eraseAllEnabled = (Boolean) cs.sketchPanelStates.get(0);
	    Boolean penUpSelected = (Boolean) cs.sketchPanelStates.get(1);
	    Boolean penUpEnabled  = (Boolean) cs.sketchPanelStates.get(2);
	    Boolean rectSelected  = (Boolean) cs.sketchPanelStates.get(3);
	    Boolean rectEnabled   = (Boolean) cs.sketchPanelStates.get(4);
	    Boolean lineSelected  = (Boolean) cs.sketchPanelStates.get(5);
	    Boolean lineEnabled   = (Boolean) cs.sketchPanelStates.get(6);
	    Boolean circSelected  = (Boolean) cs.sketchPanelStates.get(7);
	    Boolean circEnabled   = (Boolean) cs.sketchPanelStates.get(8);
	    penUpControl.setSelected(penUpSelected.booleanValue());
	    rectControl.setSelected(rectSelected.booleanValue());
	    lineControl.setSelected(lineSelected.booleanValue());
	    circControl.setSelected(circSelected.booleanValue());

	    eraseAll.setEnabled(eraseAllEnabled.booleanValue());
	    penUpControl.setEnabled(penUpEnabled.booleanValue());
	    rectControl.setEnabled(rectEnabled.booleanValue());
	    lineControl.setEnabled(lineEnabled.booleanValue());
	    circControl.setEnabled(circEnabled.booleanValue());
	}
	else {
	    penUpControl.setSelected(true);
	    rectControl.setSelected(false);
	    lineControl.setSelected(false);
	    circControl.setSelected(false);

	    enableButtons(false);
	}
	updateHighlighting();
    }

    // called by the Network Client in remote app
    public void remoteSyncEvent(RemoteSyncEvent e) {
	Util.dbgPrintln("RemoteSketchPanel::remoteSyncEvent");
	setCurrentStates(e);
    }

}
