/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;
import java.awt.*;
import java.util.*;

class StreamerVertex extends SquareVertex
{
    private long expiryTime;
    private static long lastAdd[];

    static {
	lastAdd = new long[2]; // should eventually be a hashtable or something similar
    }

    public StreamerVertex(String id, double x, double y, double z, long timeout)
    {
	super(id, x, y, z);
	super.drawCoord = false;
	long ct = System.currentTimeMillis();
	setLastAdd(id, ct);
	expiryTime = ct + timeout;
    }

    private static void setLastAdd(String id, long time) {
	int sel = id.equals("ekf-passive") ? 1 : 0;
	lastAdd[sel] = time;
    }
    
    public static long getLastAdd(String id) {
	int sel = id.equals("ekf-passive") ? 1 : 0;
	return lastAdd[sel];
    }

    protected void addList()
    {
	Vertex.streamer.add(this);
    }

    public long getExpiryTime() { return expiryTime; }

    // not thread safe!
    public static void refresh()
    {
	Iterator it  = streamer.iterator();
	long curTime = System.currentTimeMillis();
	while(it.hasNext()) {
	    StreamerVertex v = (StreamerVertex) it.next();
	    if(v.getExpiryTime() < curTime)
		it.remove();
	    else
		break; // optimization, assumes streamer has increasing time order
	}
    }
}
