/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.logging;

import javax.swing.border.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import cricketdaemon.reader.sim.CustomFrame;
import cricketdaemon.Util;

/**
 * A GUI that dumps the beacon readings onto a scroll pane.  The scroll
 * pane scrolls up automatically as new data is read.  It turns out that
 * such a feature runs extremely slowly on the iPAQ.  We will phase out the
 * GUI soon.
 *
 * @author Allen Miu
 */
class GUI extends JTextArea implements Runnable, MouseListener {

    private CustomFrame frame;
    private JButton button;
    private JScrollBar scrollBar;
    private LoggingModule ptm;

    private class TextChangeListener 
	extends JTextComponent.AccessibleJTextComponent 
    {    
	private GUI selfRef;
	public TextChangeListener(GUI g) { selfRef = g; }

	public void insertUpdate(DocumentEvent e) 
	{
	    SwingUtilities.invokeLater(selfRef);
	}
    }

    public GUI(LoggingModule ptm, int x, int y)
    {
	this.ptm = ptm;
	Dimension size = new Dimension(x,y);

	CustomFrame frame = new CustomFrame("Logging Logging");
	frame.setResizable(true);
	Container container = frame.getContentPane();
	container.setLayout(new BorderLayout());

	setLineWrap(true);
	setWrapStyleWord(true);
	setEditable(false);
	setAutoscrolls(true);

	JScrollPane scroll = 
	    new JScrollPane(this, 
			    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
	scroll.setBorder(createBorder());
	scroll.setPreferredSize(size);
	scroll.setAutoscrolls(true);
	container.add(scroll, "Center");
	Document textDoc = getDocument();
	textDoc.addDocumentListener(new TextChangeListener(this));

	scrollBar = scroll.getVerticalScrollBar();
	Util.assert_nonjava(scrollBar != null, "GUI::GUI() scrollBar null");

	button = new JButton("Separation Mark");
	button.addMouseListener(this);

	button.setBorder(BorderFactory.createRaisedBevelBorder());
	container.add(button, "South");
	
	frame.setBackground(Color.white);
	frame.validate();
	frame.pack();
	frame.setVisible(true);

	Util.log("view created and exposed");
    }

    /* MouseListener stuff */
    public void mouseClicked(MouseEvent e) {
	long curTime = System.currentTimeMillis();
	String s = LoggingModule.HASH + " " + curTime;
	dumpln(s);
	ptm.dumpln(s);
    }
    
    public void mouseEntered(MouseEvent e) { return; }
    public void mouseExited(MouseEvent e) { return; }
    public void mousePressed(MouseEvent e) { return; }
    public void mouseReleased(MouseEvent e) { return; }


    /* public API for program */
    public void dump(String s)
    {
	append(s);
    }
    
    public void dumpln(String s)
    {
	dump(s+"\n");
    }

    private Border createBorder() 
    {
	Border raisedbevel = BorderFactory.createRaisedBevelBorder();
	Border loweredbevel = BorderFactory.createLoweredBevelBorder();
	
	return
	    BorderFactory.createCompoundBorder(raisedbevel, loweredbevel);
    }

    public void run() 
    {
	scrollBar.setValue(scrollBar.getMaximum());
    }
}
