/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.logging;

import cricketdaemon.Util;
import cricketdaemon.module.*;
import cricketdaemon.reader.*;
import cricketdaemon.reader.parse.ParseMap;
import cricketdaemon.reader.PlaybackReader.*;
import java.util.*;
import java.io.*;

/**
 * A module that simple dumps the results from the Reader module.
 *
 * @author Allen Miu
 */
public class LoggingModule extends ModuleThread 
    implements Beacon.BeaconHandler
{
    public static final String HASH = "#####";

    private PrintWriter file;
    GUI gui;

    public LoggingModule(ModuleThread mt, String fileName)
    {
	super("LoggingModule",0);
	BitSet fieldMask = new BitSet();
	fieldMask.set(ParseMap.SPACE);
	fieldMask.set(ParseMap.BEACONID);
	fieldMask.set(ParseMap.DISTANCE);
	mt.register(this, fieldMask);

	gui = new GUI(this, 220,50);
	if(fileName != null)
	    initFile(fileName);
    }

    private void initFile(String fName)
    {
	try {
	    File f = new File(fName);
	    Util.assert_nonjava(!f.exists(), 
		 fName+" exists, please remove or use another filename");
	    file = 
		new PrintWriter(new FileWriter(fName, 
					       Util.DESTROY_OLDLOGFILE));
	}
	catch (IOException ie) {
	    //Util.assert_nonjava(false, "Error creating file "+f);
	}
    }

    protected void moduleCleanUp()
    {
	if(file != null) {
	    file.flush();
	    file.close();
	}
    }

    // protected void process(); // using the inherited process method

    public void processBeacon(Beacon newData, BitSet fieldMask)
    {
	//String output = System.currentTimeMillis()+" "+newData.toString2();
	String output = newData.timeStamp+" "+newData.toString2();

	//System.out.println(output);
	if(gui != null)
	    gui.dumpln(output);
	dumpln(output);
    }

    public void dumpln(String s)
    {
	System.out.println(s);
	dump(s+"\n");
    }

    public void dump(String s)
    {
	if(file != null) {
	    file.print(s);
	    file.flush();
	}
    }

    public static PlaybackBeacon parsePlayback(StringTokenizer s)
    {
	if(s.equals(HASH))
	    return null;

	long timeStamp = -1;
	try {
	    timeStamp = Long.parseLong(s.nextToken());
	}
	catch (NoSuchElementException e) {
	    Util.log("LoggingModule::parsePlayback() invalid line");
	    return null;
	}
	catch (NumberFormatException ie) {
	    Util.log("LoggingModule::parsePlayback() invalid line n");
	    return null;
	}

	PlaybackBeacon result = Beacon.parsePlayback(s, timeStamp);
	if(result != null)
	    result.timeStamp = timeStamp;
	return result;
    }
}
